;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Indent-tabs-mode: NIL -*-
;;;
;;; timers.lisp --- Creating and manipulating timer structures.
;;;
;;; Copyright (C) 2003 Zach Beane <xach@xach.com>
;;;
;;; Permission is hereby granted, free of charge, to any person obtaining
;;; a copy of this software and associated documentation files (the
;;; "Software"), to deal in the Software without restriction, including
;;; without limitation the rights to use, copy, modify, merge,publish,
;;; distribute, sublicense, and/or sell copies of the Software, and to
;;; permit persons to whom the Software is furnished to do so, subject to
;;; the following conditions:
;;;
;;; The above copyright notice and this permission notice shall be
;;; included in all copies or substantial portions of the Software.
;;;
;;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
;;; NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
;;; LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
;;; OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
;;; WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(in-package :bordeaux-timers)


;;;
;;; Timer
;;;

(defstruct (timer
             (:conc-name %timer-)
             (:constructor %make-timer)
             (:print-function %print-timer))
  name
  function
  expire-time
  repeat-time
  thread)

(defun make-timer (function &key name (thread t))
  (%make-timer :name name :function function :thread thread))

(defun %print-timer (object stream print-level)
  (declare (ignore print-level))
  (print-unreadable-object (object stream)
    (format stream "TIMER ~A" (or (%timer-name object)
                                  "(unnamed)"))))

(defun timer-name (timer)
  (%timer-name timer))

(defun timer-expired-p (timer &optional (delta 0.0))
  (or (null (%timer-expire-time timer))
      (let ((compare-time (+ (get-precise-time) delta)))
        (> compare-time (%timer-expire-time timer)))))
